@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1982-2014 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2024 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -128,7 +128,7 @@ void	sh_fault(int sig)
 			sigrelease(sig);
 			if(pp->mode != SH_JMPSUB)
 			{
-				if(pp->mode < SH_JMPSUB)
+				if(pp->mode < SH_JMPSUB && !sh_isstate(SH_INTERACTIVE))
 					pp->mode = sh.subshell?SH_JMPSUB:SH_JMPFUN;
 				else
 					pp->mode = SH_JMPEXIT;
@@ -495,7 +495,7 @@ int sh_trap(const char *trap, int mode)
 	char	was_no_trapdontexec = !sh.st.trapdontexec;
 	char	save_chldexitsig = sh.chldexitsig;
 	int	staktop = stktell(sh.stk);
-	char	*savptr = stkfreeze(sh.stk,0);
+	void	*savptr = stkfreeze(sh.stk,0);
 	struct	checkpt buff;
 	Fcin_t	savefc;
 	fcsave(&savefc);
@@ -526,11 +526,7 @@ int sh_trap(const char *trap, int mode)
 		if(jmpval==SH_JMPSCRIPT)
 			indone=0;
 		else
-		{
-			if(jmpval==SH_JMPEXIT)
-				savxit = sh.exitval;
 			jmpval=SH_JMPTRAP;
-		}
 	}
 	sh_popcontext(&buff);
 	/* re-allow last-command exec optimisation unless the command we executed set a trap */
@@ -539,8 +535,10 @@ int sh_trap(const char *trap, int mode)
 	sh.intrap--;
 	sfsync(sh.outpool);
 	savxit_return = sh.exitval;
-	if(jmpval!=SH_JMPEXIT && jmpval!=SH_JMPFUN)
-		sh.exitval=savxit;
+	if(sh.intrap_exit_n)
+		sh.intrap_exit_n = 0;
+	else
+		sh.exitval = savxit;
 	stkset(sh.stk,savptr,staktop);
 	fcrestore(&savefc);
 	if(was_history)
@@ -571,6 +569,7 @@ void sh_exit(int xno)
 		sh.exitval |= (sig=sh.lastsig);
 	if(pp && pp->mode>1)
 		cursig = -1;
+	sh_offstate(SH_EXEC);
 	if((sh.trapnote&SH_SIGTSTP) && job.jobcontrol)
 	{
 		/* ^Z detected by the shell */
@@ -617,7 +616,7 @@ void sh_exit(int xno)
 	}
 	/* unlock output pool */
 	sh_offstate(SH_NOTRACK);
-	if(!(pool=sfpool(NULL,sh.outpool,SF_WRITE)))
+	if(!(pool=sfpool(NULL,sh.outpool,SFIO_WRITE)))
 		pool = sh.outpool; /* can't happen? */
 	sfclrlock(pool);
 	if(sh.lastsig==SIGPIPE)
@@ -693,7 +692,7 @@ noreturn void sh_done(int sig)
 		/* generate fault termination code */
 		if(RLIMIT_CORE!=RLIMIT_UNKNOWN)
 		{
-#ifdef _lib_getrlimit
+#if _lib_getrlimit
 			struct rlimit rlp;
 			getrlimit(RLIMIT_CORE,&rlp);
 			rlp.rlim_cur = 0;
