@@ -2,7 +2,7 @@
 *                                                                      *
 *               This software is part of the ast package               *
 *          Copyright (c) 1992-2012 AT&T Intellectual Property          *
-*          Copyright (c) 2020-2022 Contributors to ksh 93u+m           *
+*          Copyright (c) 2020-2023 Contributors to ksh 93u+m           *
 *                      and is licensed under the                       *
 *                 Eclipse Public License, Version 2.0                  *
 *                                                                      *
@@ -25,7 +25,7 @@
 #include <cmd.h>
 
 int
-b_mv(int argc, register char** argv, Shbltin_t* context)
+b_mv(int argc, char** argv, Shbltin_t* context)
 {
 	return b_cp(argc, argv, context);
 }
